#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE

# Copyright (C) 2013 National University of Defense Technology(NUDT) & Kylin Ltd

# Author:     
#     kobe Lee <xiangli@ubuntukylin.com>
#     maclin <majun@ubuntukylin.com>
# Maintainer:
#     maclin <majun@ubuntukylin.com>

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

### END LICENSE

import sys
import os
import signal
import glob
import fcntl
import shutil
# import logging
import tempfile
import subprocess
import re
import dbus
import dbus.service
import sqlite3
import dbus.mainloop.glib
from gi.repository import GObject
from PyQt5.QtCore import *
import apt
import aptsources.sourceslist
import apt_pkg
import time
import fcntl
from kylin_software_center_daemon.aescrypt import *

from kylin_software_center_daemon.apt_daemon import AptDaemon, AppActions
from kylin_software_center_daemon.apt_daemon import WorkitemError

import threading
lock = threading.Lock()
# log = logging.getLogger('Daemon')

INTERFACE = 'com.kylin.softwarecenter'
UKPATH = '/'

HTTP_SOURCE_UBUNTUKYLIN = "http://archive.ubuntukylin.com:10006/ubuntukylin"
DEB_SOURCE_UBUNTUKYLIN = "deb " + HTTP_SOURCE_UBUNTUKYLIN
UBUNTUKYLIN_SOFTWARECENTER_ACTION = 'com.kylin.softwarecenter.action'

LIB_PATH = "/var/lib/"
DPKG_PATH = "dpkg/lock"


class WorkItem:
    def __init__(self, pkgname, action, kwargs):
        self.pkgname = pkgname
        self.action = action
        self.kwargs = kwargs


class WorkThread(threading.Thread):

    def __init__(self, dbus_service):
        threading.Thread.__init__(self)
        self.dbus_service = dbus_service
        self.thread_is_working = 0
        self.uksc_is_working = 0
        self.uksc_exit = False

    def run(self):
        #        print "The backend start the work thread..."
        while (True):
            if self.thread_is_working < 100:
                self.thread_is_working += 1
            else:
                self.thread_is_working = 0
            if len(self.dbus_service.worklist) == 0:
                if(self.uksc_exit == True):
                    self.dbus_service.mainloop.quit()
                    break
                else:
                    time.sleep(0.5)
                    continue

            if is_file_locked(LIB_PATH + DPKG_PATH) is True or 1 == self.uksc_is_working:
                time.sleep(0.5)
                continue

            self.dbus_service.mutex.acquire()
            item = self.dbus_service.worklist.pop(0)  # pop(0) is get first item and remove it from list
            self.dbus_service.mutex.release()
            self.uksc_is_working = 1

            try:
                func = getattr(self.dbus_service.daemonApt, item.action)
                if func is None:
                    if(self.DEBUG_SWITCH):
                        print("Error action: ", item)

                res = func(item.pkgname, item.kwargs)
                if res is False:
                    if(self.DEBUG_SWITCH):
                        print("Action exec failed...")
            except WorkitemError as e:
                # print(e.errornum)
                self.uksc_is_working = 0
                kwarg = {"apt_appname": item.pkgname,
                         "apt_percent": str(-e.errornum),
                         "action": str(item.action),
                         }
                self.dbus_service.software_apt_signal("apt_error", kwarg)
            except:
                self.uksc_is_working = 0
                kwarg = {"apt_appname": item.pkgname,
                         "apt_percent": str(-6.6),
                         "action": str(item.action),
                         }
                self.dbus_service.software_apt_signal("apt_error", kwarg)


#            print "finish one acion....."
# time.sleep(0.5)


def is_file_locked(lockfile):
    """
    Check whether ``apt-get`` or ``dpkg`` is currently active by check the lock file.

    This works by checking whether the lock file like ``/var/lib/
    dpkg/lock``
    ``/var/lib/apt/lists/lock`` is locked by an ``apt-get`` or ``dpkg`` process,
    which in turn is done by momentarily trying to acquire the lock.
     This means that the current process needs to have sufficient privileges.

    :returns: ``True`` when the lock is already taken (``apt-get`` or ``dpkg``
              is running), ``False`` otherwise.
    :raises: :py:exc:`exceptions.IOError` if the required privileges are not
             available.

    .. note:: ``apt-get`` doesn't acquire this lock until it needs it, for
              example an ``apt-get update`` run consists of two phases (first
              fetching updated package lists and then updating the local
              package index) and only the second phase claims the lock (because
              the second phase writes the local package index which is also
              read from and written to by ``dpkg``).
    """
    with open(lockfile, 'w') as handle:
        try:
            fcntl.lockf(handle, fcntl.LOCK_EX | fcntl.LOCK_NB)
            return False
        except IOError:
            return True


class SoftwarecenterDbusService(dbus.service.Object):

    DE_STRING = "LS0tLS1CRUdJTiBQVUJMSUMgS0VZLS0tLS0KTUlHZk1BMEdDU3FHU0liM0RRRUJBUVVBQTRHTkFEQ0JpUUtCZ1FDVDE4YmxXYmtLY0hSR2cyZ20zU2xRT3dJcQorVE42OXFBK0RHblprMWdVTXQ2Zk5LWnkwVFRMWCt4YzFGTE9JZHQxVlorU01rYW9taWROMER4eThJV0tzYnErCnlpUDFJTG1DbDd2S0hNNXpDSFBqSmxYWHJSdFdZNUZCbnFYcFpNbHBpV0xRdWxpZStjOUlqb1J4ajR2eUprQ28KNUV5Qk1aSlliR2l0TGpobHB3SURBUUFCCi0tLS0tRU5EIFBVQkxJQyBLRVktLS0tLQo="

    def __init__(self, bus, mainloop):
        self.DEBUG_SWITCH = False
        self.daemonApt = AptDaemon(self)
        self.bus = bus
        self.bus_name = dbus.service.BusName(INTERFACE, bus=bus)
        #        print "SoftwarecenterDbusService:",self.bus_name
        dbus.service.Object.__init__(self, self.bus_name, UKPATH)
        self.mainloop = mainloop
        self.worklist = []
        self.cancel_name_list = []
        self.mutex = threading.RLock()
        self.cancelmutex = threading.RLock()
        self.worker_thread = WorkThread(self)
        self.worker_thread.setDaemon(True)
        self.worker_thread.start()

    #
    # 函数：操作policykit提权
    #
    @dbus.service.method(INTERFACE, in_signature='sss', out_signature='b', sender_keyword='sender')
    def auth_with_policykit(self, text,get_action_id,signature_data, sender=None):
        res = QByteArray()
        res.append(text)
        res.append(get_action_id)
        cryptographichash = QCryptographicHash(QCryptographicHash.Md5)
        cryptographichash.addData(res)
        md5 = cryptographichash.result()
        aescryptor = Aescrypt()  # ECB模式
        is_verify = aescryptor.add_key_dncrypt(base64.b64decode(self.DE_STRING), signature_data.encode(), md5.data())
        # return True
        if(is_verify == True):
            action = get_action_id
            if text == "":
                text = "要安装或卸载软件"
            if not sender:
                raise ValueError('sender == None')

            #        print "auth_with_policykit:", sender
            granted = False
            try:
                obj = dbus.SystemBus().get_object('org.freedesktop.PolicyKit1',
                                                  '/org/freedesktop/PolicyKit1/Authority')
                policykit = dbus.Interface(obj, 'org.freedesktop.PolicyKit1.Authority')
                subject = ('system-bus-name', {'name': sender})
                flags = dbus.UInt32(1)  # AllowUserInteraction flag
                msg = text + "，您需要进行验证。"
                details = {'polkit.message': msg}
                cancel_id = ''  # No cancellation id
                (granted, notused, details) = policykit.CheckAuthorization(
                    subject, action, details, flags, cancel_id)
            except:
                print("auth with except......")
                granted = False

            return granted
        else:
            return False

    # @dbus.service.method(INTERFACE, in_signature='s', out_signature='b')
    def run_app(self,cmd):
        user = os.popen("who|sed -e 's/[[:space:]][[:space:]]*/ /g'|awk '{print $1}'").readline().strip("\n").strip("\t")
        try:
            os.system("su - "+user+" -c '"+ cmd +"'" )
        except:
            return False
        return True


    #
    # 函数：添加线程任务
    #
    def add_worker_item(self, item):
        if(self.DEBUG_SWITCH):
            print("####add_worker_item:", item)
        self.mutex.acquire()
        self.worklist.append(item)
        self.mutex.release()
        if (self.DEBUG_SWITCH):
            print("####add_worker_item finished!")

    #
    # 函数：删除线程任务
    #
    def del_worker_item_by_name(self, cancelinfo):
        if (self.DEBUG_SWITCH):
            print("####del_worker_item_by_name:", cancelinfo[0])

        del_work_item = None
        self.mutex.acquire()
        try:
            for item in self.worklist:
                if item.pkgname == cancelinfo[0] and item.action == cancelinfo[1]:
                    self.worklist.remove(item)
                    del_work_item = item
                    break
        except:
            pass
        self.mutex.release()

        self.cancelmutex.acquire()
        if del_work_item != None:
            self.cancel_name_list.append(del_work_item)
        self.cancelmutex.release()
        if (self.DEBUG_SWITCH):
            print("####del_worker_item_by_name finished!")

    def check_cancel_worker_item(self, cancelinfo):
        cancel = False
        self.cancelmutex.acquire()
        #        print "check_cancel_worker_item:",len(self.cancel_name_list)

        for item in self.cancel_name_list:
            if item.pkgname == cancelinfo[0] and item.action == cancelinfo[1]:
                self.cancel_name_list.remove(item)
                cancel = True
                break
        self.cancelmutex.release()

        #        print "####check_cancel_worker_item finished!:",cancel
        return cancel

    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def uksc_exit(self):
        self.worker_thread.uksc_exit = True

    @dbus.service.method(INTERFACE, in_signature='', out_signature='b')
    def check_apt_lock(self):
        result = os.popen("ps -u root | awk '{print $4}'").readlines()
        for i in result:
            if i.strip("\n") == "apt" or i.strip("\n") == "apt-get" or i.strip("\n") == "dpkg":
                return False
            #当遍历到最后一个时
            if i == result[len(result) - 1]:
                if os.path.exists("/var/cache/apt/archives/lock"):
                    os.remove("/var/cache/apt/archives/lock")
                elif os.path.exists("/var/lib/dpkg/lock"):
                    os.remove("/var/lib/dpkg/lock")
                os.system("dpkg --configure -a")
                os.system("apt-get install -f -y")
                return True


    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def exit(self):
        self.mainloop.quit()

    # lixiang: wakeup dbus with timeout
    @dbus.service.method(INTERFACE, in_signature='', out_signature='s')
    def wakeup(self):
        return "kylin"

    # check ubuntukylin source is in /etc/apt/sources.list or not
    #@dbus.service.method(INTERFACE, in_signature='', out_signature='b', sender_keyword='sender')
    # def check_source_ubuntukylin(self, sender=None):
    #     source = aptsources.sourceslist.SourcesList()
    #     for item in source.list:
    #         if (item.str().find(DEB_SOURCE_UBUNTUKYLIN) != -1):
    #             return True
    #     return False

    # add ubuntukylin source in /etc/apt/sources.list
    #@dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    # def add_source_ubuntukylin(self, version, sender=None):
    #
    #     granted = self.auth_with_policykit("要增加软件源","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         return False
    #
    #     source = aptsources.sourceslist.SourcesList(())
    #     # ????the check option should include version
    #     if (self.check_source_ubuntukylin() is True):
    #         return False
    #     osversion = str(version) + (" main")
    #     source.add("deb", HTTP_SOURCE_UBUNTUKYLIN, osversion, "")
    #     source.save()
    #     return True

    # add source in /etc/apt/sources.list
    #@dbus.service.method(INTERFACE, in_signature='s', out_signature='s', sender_keyword='sender')
    # def add_source(self, text, sender=None):
    #
    #     granted = self.auth_with_policykit("要增加软件源","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         return '0'
    #
    #     return self.daemonApt.add_source(text)

    # remove source from /etc/apt/sources.list
    #@dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    # def remove_source(self, text, sender=None):
    #
    #     granted = self.auth_with_policykit("","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         return False
    #
    #     return self.daemonApt.remove_source(text)

    # check ubuntukylin source is in /etc/apt/sources.list or not
    #@dbus.service.method(INTERFACE, in_signature='b', out_signature='as', sender_keyword='sender')
    # def get_sources(self, except_ubuntu, sender=None):
    #
    #     return self.daemonApt.get_sources(except_ubuntu)

    # -------------------------software-center-------------------------

    # install deb file
    #@dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    # def install_debfile(self, path, sender=None):
    #     if (self.DEBUG_SWITCH):
    #         print("####install deb file: ", path)
    #     # path = "".join([chr(character) for character in path]) # add by zhangxin for chinese .deb path 11.19
    #     granted = self.auth_with_policykit("","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         kwarg = {"appname": path,
    #                  "action": AppActions.INSTALLDEBFILE,
    #                  }
    #         self.software_auth_signal("auth_cancel", kwarg)
    #         return False
    #
    #     item = WorkItem(path, AppActions.INSTALLDEBFILE, None)
    #     self.add_worker_item(item)
    #     return True

    # install deb file's deps
    #@dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    # def install_deps(self, path, sender=None):
    #     if (self.DEBUG_SWITCH):
    #         print("####install deps: ", path)
    #     # path = "".join([chr(character) for character in path])
    #     granted = self.auth_with_policykit("", "com.kylin.softwarecenter.action",sender)
    #     if not granted:
    #         kwarg = {"appname": path,
    #                  "action": AppActions.INSTALLDEPS,
    #                  }
    #         self.software_auth_signal("auth_cancel", kwarg)
    #         return False
    #
    #     item = WorkItem(path, AppActions.INSTALLDEPS, None)
    #     self.add_worker_item(item)
    #     return True

    # install package sa:software_fetch_signal() and software_apt_signal()
    @dbus.service.method(INTERFACE, in_signature='ss', out_signature='b', sender_keyword='sender')
    def install(self, pkgName, signature_data, sender=None):
        res = QByteArray()
        res.append(pkgName)
        cryptographichash = QCryptographicHash(QCryptographicHash.Md5)
        cryptographichash.addData(res)
        md5 = cryptographichash.result()
        aescryptor = Aescrypt()  # ECB模式
        is_verify = aescryptor.add_key_dncrypt(base64.b64decode(self.DE_STRING), signature_data.encode(), md5.data())
        if(is_verify == True):
            if (self.DEBUG_SWITCH):
                print("####install: ", pkgName)

            item = WorkItem(pkgName, AppActions.INSTALL, None)

            self.add_worker_item(item)

            #        self.daemonApt.install_pkg(pkgName)
            if (self.DEBUG_SWITCH):
                print("####install return")
            return True
        else:
            return False

    # uninstall package sa:software_apt_signal()
    @dbus.service.method(INTERFACE, in_signature='ss', out_signature='b', sender_keyword='sender')
    def remove(self, pkgName,signature_data, sender=None):
        res = QByteArray()
        res.append(pkgName)
        cryptographichash = QCryptographicHash(QCryptographicHash.Md5)
        cryptographichash.addData(res)
        md5 = cryptographichash.result()
        aescryptor = Aescrypt()  # ECB模式
        is_verify = aescryptor.add_key_dncrypt(base64.b64decode(self.DE_STRING), signature_data.encode(), md5.data())
        if(is_verify):
            list = []
            user = os.popen("who|sed -e 's/[[:space:]][[:space:]]*/ /g'|awk '{print $1}'").readline()
            if(user.strip("\n").strip(" ").strip("\t") != "root"):
                destFile = "/home/"+user.strip("\n").strip(" ").strip("\t")+"/.cache/uksc/uksc.db"
            else:
                destFile = "/" + user.strip("\n").strip(" ").strip("\t")+"/.cache/uksc/uksc.db"
            try:
                self.connect = sqlite3.connect(destFile, timeout=30.0, check_same_thread=False)
                self.cursor = self.connect.cursor()
                try:
                    lock.acquire(True)
                    self.cursor.execute("select app_name from application")
                    res = self.cursor.fetchall()
                finally:
                    lock.release()
                if len(res) == 0:
                    list = []
                else:
                    list = res
            except Exception as e:
                return False
            in_list = False
            for i in list:
                if(i[0] == pkgName):
                    in_list = True
            if(in_list == False):
                return False
            if (self.DEBUG_SWITCH):
                print("####remove: ", pkgName)

            item = WorkItem(pkgName, AppActions.REMOVE, None)

            self.add_worker_item(item)

            #        self.daemonApt.uninstall_pkg(pkgName)
            if (self.DEBUG_SWITCH):
                print("####remove return")
            return True
        else:
            return False

    # update package sa:software_fetch_signal() and software_apt_signal()
    #@dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
    # def upgrade(self, pkgName, sender=None):
    #     if (self.DEBUG_SWITCH):
    #         print("####upgrade: ", pkgName)
    #
    #     granted = self.auth_with_policykit("","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         kwarg = {"appname": pkgName,
    #                  "action": AppActions.UPGRADE,
    #                  }
    #         self.software_auth_signal("auth_cancel", kwarg)
    #         return False
    #
    #     item = WorkItem(pkgName, AppActions.UPGRADE, None)
    #
    #     self.add_worker_item(item)
    #
    #     #        self.daemonApt.upgrade_pkg(pkgName)
    #     if (self.DEBUG_SWITCH):
    #         print("####upgrade return")
    #     return True

    #@dbus.service.method(INTERFACE, in_signature='as', out_signature='s', sender_keyword='sender')
    # def cancel(self, cancelinfo, sender=None):
    #     if (self.DEBUG_SWITCH):
    #         print("####cancel: ", cancelinfo[0])
    #
    #     granted = self.auth_with_policykit("","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         return "False"
    #
    #     self.del_worker_item_by_name(cancelinfo)
    #     if self.check_cancel_worker_item(cancelinfo) is True:
    #         if (self.DEBUG_SWITCH):
    #             print("####cancel return")
    #         return "True"
    #     else:
    #         return "False"

    # apt-get update sa:software_fetch_signal()
    #z@dbus.service.method(INTERFACE, in_signature='b', out_signature='s', sender_keyword='sender')
    # def update(self, quiet, sender=None):
    #     if (self.DEBUG_SWITCH):
    #         print("####update: ")
    #
    #     granted = self.auth_with_policykit("要更新软件源","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         return "False"
    #
    #     if is_file_locked("/var/lib/apt/lists/lock") is True:
    #         return "Locked"
    #
    #     kwargs = {"quiet": str(quiet),
    #               }
    #
    #     item = WorkItem("#update", AppActions.UPDATE, kwargs)
    #
    #     self.add_worker_item(item)
    #
    #     #        self.daemonApt.update()
    #     # print "####update return"
    #     return "True"

    # ????????????????????????????
    # apt-get update sa:software_fetch_signal()
    #@dbus.service.method(INTERFACE, in_signature='b', out_signature='s', sender_keyword='sender')
    # def update_first(self, quiet, sender=None):
    #     if (self.DEBUG_SWITCH):
    #         print("####update first: ")
    #
    #     granted = self.auth_with_policykit( "要更新软件源","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         return "False"
    #
    #     kwargs = {"quiet": str(quiet),
    #               }
    #
    #     item = WorkItem("#update", AppActions.UPDATE_FIRST, kwargs)
    #
    #     self.add_worker_item(item)
    #
    #     #        self.daemonApt.update()
    #
    #     return "True"

    # check packages status by pkgNameList sa:software_check_status_signal()
    @dbus.service.method(INTERFACE, in_signature='as', out_signature='')
    def check_pkgs_status(self, pkgNameList):
        self.daemonApt.check_pkgs_status_rtn_list(pkgNameList)

    # check one package status by pkgName
    @dbus.service.method(INTERFACE, in_signature='s', out_signature='s')
    def check_pkg_status(self, pkgName):
        return self.daemonApt.check_pkg_status(pkgName)

    # ------------------get info about apt depend----------add 20140926----
    def _common_get_marked_cache(self, pkgName=None, cache=None):
        """"Return the cache that pkg had been marked."""
        if cache is None:
            cache = apt.cache.Cache()
        else:
            cache.clear()

        if pkgName is None:
            return False
        pkg = cache[pkgName]

        if pkg.is_installed:
            pkg.mark_delete()
        else:
            pkg.mark_install()

        return cache

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='i')
    def require_totally_pkg_size(self, pkgName=None):
        """Return the size of the additional reuquired space on the fs."""
        markedCache = self._common_get_marked_cache(pkgName, )
        if not markedCache:
            return False

        # markedCache.clear()
        return markedCache.required_space

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='i')
    def download_totally_pkg_size(self, pkgName=None):
        """Return the size of the package that are required to download."""
        markedCache = self._common_get_marked_cache(pkgName, )
        if not markedCache:
            return False

        return markedCache.required_download

    @dbus.service.method(INTERFACE, in_signature='s', out_signature='')
    def require_totally_pkg_list(self, pkgName=None):
        """Return a dictionary about dep-pkg should been install or delete """
        dic = {"install": [], "update": [], "delete": [], "broken": []}
        markedCache = self._common_get_marked_cache(pkgName, )
        if not markedCache:
            return False

        temp = markedCache.get_changes()
        for i in temp:
            if (i.marked_install):
                dic["install"].append(i.name)
            elif (i.marked_delete):
                dic["delete"].append(i.name)
            elif (i.marked_upgrade):
                if (i.is_installed):
                    dic["update"].append(i.name)
            elif (i.is_inst_broken):
                dic["broken"].append(i.name)
        if (self.DEBUG_SWITCH):
            print(dic)
        # return dic

    # package download status signal
    '''parm mean
        type:
            start:start download
            stop:all work is finish
            done:all items download finished
            fail:download failed
            fetch:one item download finished
            pulse:download status, this msg given a string like dict
        msg:
            a message of type, sometimes is None
    '''

    @dbus.service.signal(INTERFACE, signature='sa{ss}')
    def software_fetch_signal(self, type, msg):
        pass

    @dbus.service.signal(INTERFACE, signature='sa{ss}')
    def software_auth_signal(self, type, msg):
        pass

    # package install/update/remove signal
    '''parm mean
        type:
            start:start work
            stop:work finish
            error:got a error
            pulse:work status, this msg given a string like dict
        msg:
            a message of type, sometimes is None
    '''

    @dbus.service.signal(INTERFACE, signature='sa{ss}')
    def software_apt_signal(self, type, msg):
        pass

    # get packages status signal
    '''parm mean
        dict{packageName, packageStatus}
        packageStatus:
            i:installed
            u:installed and can update
            n:notinstall
    '''

    @dbus.service.signal(INTERFACE, signature='as')
    def software_check_status_signal(self, statusList):
        pass

    @dbus.service.signal(INTERFACE, signature='s')
    def software_signal_test(self, msg):
        pass

    @dbus.service.method(INTERFACE, in_signature='', out_signature='ai')
    def check_work_item(self):
        dpkg_is_running = 0
        workitemcount = 0
        self.mutex.acquire()
        if len(self.worklist) != 0:
            workitemcount = len(self.worklist)
        self.mutex.release()
        dpkg_is_running = is_file_locked(LIB_PATH + DPKG_PATH)
        if dpkg_is_running is True:
            dpkg_is_running = 1
        else:
            dpkg_is_running = 0
        res = [workitemcount, dpkg_is_running]
        return res

    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def clear_all_work_item(self):
        self.worklist = []
        self.cancel_name_list = []
        self.worker_thread.uksc_is_working = 0

    #@dbus.service.method(INTERFACE, in_signature='', out_signature='i')
    # def check_dbus_thread_is_working(self):
    #     return self.worker_thread.thread_is_working

    #@dbus.service.method(INTERFACE, in_signature='', out_signature='i')
    # def check_uksc_is_working(self):
    #     return self.worker_thread.uksc_is_working

    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def set_uksc_not_working(self):
        self.worker_thread.uksc_is_working = 0

    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def exit(self):
        self.mainloop.quit()

    # @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    # def check_dpkg_statu(self):
    #     os.system("dpkg --configure -a")

    # @dbus.service.method(INTERFACE, in_signature='', out_signature='b', sender_keyword='sender')
    # def kydroid_policykit(self, sender=None):
    #     granted = self.auth_with_policykit("移动应用操作","com.kylin.softwarecenter.action", sender)
    #     if not granted:
    #         return False
    #     else:
    #         return True

    @dbus.service.method(INTERFACE, in_signature='sssbs', out_signature='')
    def copy_file_to_install(self, app_fullname ,src_path,install_appname,need_emit_signal,signature_data):
        if not os.path.exists(src_path):
            return
        res = QByteArray()
        res.append(app_fullname)
        cryptographichash = QCryptographicHash(QCryptographicHash.Md5)
        cryptographichash.addData(res)
        md5 = cryptographichash.result()
        aescryptor = Aescrypt()  # ECB模式
        is_verify = aescryptor.add_key_dncrypt(base64.b64decode(self.DE_STRING), signature_data.encode(), md5.data())
        if (is_verify == True):
            if(app_fullname.endswith(".deb")):
                dest_path = " /var/cache/apt/archives/"
                cmd = "mv " + src_path + "/"+ app_fullname + dest_path
                os.system(cmd)
                if(need_emit_signal):
                    self.copy_finish(app_fullname,install_appname)

    # @dbus.service.method(INTERFACE, in_signature='ssss', out_signature='')
    # def copy_apk_to_install(self, app_fullname, src_path,dest_path,install_appname):
    #     cmd = "mv " + src_path + "/" + app_fullname +" "+ dest_path
    #     os.system(cmd)
    #     self.copy_finish(app_fullname,install_appname)

    #
    # 函数:判断拷贝结束
    #
    @dbus.service.signal(INTERFACE, signature="ss")
    def copy_finish(self, app_fullname,install_appname):
        pass

    # @dbus.service.method(INTERFACE, in_signature='s', out_signature='')
    # def mkdir(self,path):
    #     try:
    #         os.remove(path)
    #     except:
    #         pass
    #     try:
    #         os.mkdir(path)
    #         os.system("chmod 777 "+path)
    #     except:
    #         pass

if __name__ == '__main__':
    os.environ["TERM"] = "xterm"
    os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X11R6/bin"
    os.environ["DEBIAN_FRONTEND"] = "noninteractive"
    if os.path.exists("/var/lib/apt/lists/lock"):
        os.remove("/var/lib/apt/lists/lock")
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    GObject.threads_init()
    mainloop = GObject.MainLoop()
    signal.signal(signal.SIGINT, lambda: mainloop.quit())
    SoftwarecenterDbusService(dbus.SystemBus(), mainloop)
    mainloop.run()



#!/usr/bin/python3
# -*- coding: utf-8 -*-


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from models.enums import BUTTON_ENTER,BUTTON_LEAVE,BUTTON_PRESSED,BUTTON_DISABLE,AppMsg,ClickCard
from ui.popular_app_card import *
from models.application import *
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext


class Popular_Applications_Widget(QWidget):
    app_num = 0
    card_list = []
    ad_move_left_time = QTimer()
    ad_move_right_time = QTimer()
    final_x_position = 0
    final_x_position2 = 0
    click_card = pyqtSignal(Application,bool,int)
    def __init__(self,w,mw,parent=None):
        QWidget.__init__(self,parent)
        self.m_parent = parent
        self.appmsg = None
        self.w = w
        self.mw = mw
        self.card_width = 288
        self.card_height = 190
        self.num = 3
        self.m_parent = parent
        self.init_ui()
        self.init_connect()
        self.init_style()
        self.add_card_list = []
        self.show_card_list = []

    def init_ui(self):
        self.resize(self.w,240)
        self.setFocusPolicy(Qt.NoFocus)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.locale_lan = QLocale.system().name()
        self.titel_label = QLabel(self)
        self.titel_label.setAttribute(Qt.WA_TranslucentBackground)
        if self.locale_lan == "zh_CN":
            self.titel_label.setFixedSize(150,26)
        else:
            self.titel_label.setFixedSize(160,36)
        self.titel_label.setText(_("Popular apps"))
        self.titel_label.move(5,0)

        self.more_btn = QPushButton(self)
        self.more_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.more_btn.setFocusPolicy(Qt.NoFocus)
        self.more_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;}")
        self.more_btn.setText(_("more"))
        self.more_btn.adjustSize()
        self.more_btn.move(self.width() - self.more_btn.width(),
                           self.titel_label.y() + int((self.titel_label.height() - self.more_btn.height()) / 2))
        self.more_btn.show()

        self.app_list_widget = QWidget(self)
        self.app_list_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.app_list_widget.setFixedHeight(self.height()-self.titel_label.y()-self.titel_label.height())
        self.app_list_widget.move(0,self.titel_label.height())

        self.app_list_widget2 = QWidget(self)
        self.app_list_widget2.setFixedHeight(self.height()-self.titel_label.y()-self.titel_label.height())
        self.app_list_widget2.setAttribute(Qt.WA_TranslucentBackground)
        self.app_list_widget2.move(0,  self.titel_label.height())

        self.left_btn = QPushButton(self)
        self.left_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.left_btn.installEventFilter(self)
        self.left_btn.setFocusPolicy(Qt.NoFocus)
        self.left_btn.resize(60,60)
        if(Globals.THEME_COLOR == 0):
            self.left_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:30px;border-image: url(./res/ads_left1.png);}QPushButton:hover{border-image: url(./res/ads_left2.png);}QPushButton:pressed{border-image: url(./res/ads_left2.png);}")
        else:
            self.left_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:30px;border-image: url(./res/ads_left_dark1.png);}QPushButton:hover{border-image: url(./res/ads_left_dark2.png);}QPushButton:pressed{border-image: url(./res/ads_left_dark2.png);}")
        self.left_btn.move(6,int((self.height()-self.left_btn.height())/2)+21)
        self.left_btn.hide()

        self.right_btn = QPushButton(self)
        self.right_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.right_btn.installEventFilter(self)
        self.right_btn.setFocusPolicy(Qt.NoFocus)
        self.right_btn.resize(60, 60)
        self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:30px;border-image: url(./res/ads_right1.png);}QPushButton:hover{border-image: url(./res/ads_right2.png);}QPushButton:pressed{border-image: url(./res/ads_right2.png);}")
        self.right_btn.move(self.width()-self.right_btn.width()-6,int((self.height()-self.left_btn.height())/2)+21)
        self.right_btn.hide()

    def re_move(self):
        self.left_btn.move(6, int((self.height() - self.left_btn.height()) / 2) + 21)
        self.right_btn.move(self.width() - self.right_btn.width()-6,
                            int((self.height() - self.left_btn.height()) / 2) + 21)
        if (self.app_list_widget.width() < self.width()):
            self.app_list_widget.move(0,self.app_list_widget.y())
            self.app_list_widget2.hide()
        if(int(self.width()/(self.card_width+20)) <3 and int(self.app_list_widget.width()/(self.card_width+20)) > 0):
            self.num = int(self.width()/(self.card_width+20))
        else:
            self.num = 3

    def init_style(self):
        self.titel_label.setStyleSheet("QLabel{font-size: 24px;font-weight: 500;color: #303133;line-height: 33px;}")

    def get_lists(self, applist):
        self.applist = []
        self.applist.extend(applist)

    def init_connect(self):
        self.right_btn.clicked.connect(self.slot_click_right_btn)
        self.left_btn.clicked.connect(self.slot_click_left_btn)
        self.ad_move_left_time.timeout.connect(self.slot_slide_left)
        self.ad_move_right_time.timeout.connect(self.slot_slide_right)

    #
    #函数：添加界面到热门应用
    #
    def add_app_widget(self,app_msg,card_path):
        itemwidget = Popular_Card_Widget(app_msg,self.card_width,self.card_height,card_path, self.app_list_widget)
        itemwidget2 = Popular_Card_Widget(app_msg, self.card_width,self.card_height,card_path,self.app_list_widget2)
        itemwidget.clicked_card.connect(self.slot_click_card)
        itemwidget2.clicked_card.connect(self.slot_click_card)
        self.add_card_list.append(app_msg.app_id)
        #self.card_list.append(app_widget)
        if(self.app_num == 0):
            itemwidget.move(5,15)
            itemwidget2.move(5,15)
            self.app_list_widget.setFixedWidth(itemwidget.width()+10)
            self.app_list_widget2.setFixedWidth(itemwidget.width()+10)
            self.app_list_widget2.move(self.app_list_widget.x()+self.app_list_widget.width(),self.titel_label.height())
        else:
            itemwidget.move(self.app_num*(itemwidget.width()+20)+5,15)
            itemwidget2.move(self.app_num*(itemwidget.width()+20)+5,15)
            self.app_list_widget.setFixedWidth(self.app_num*(itemwidget.width()+20)+itemwidget.width()+10)
            self.app_list_widget2.setFixedWidth(self.app_num*(itemwidget.width()+20)+itemwidget.width()+10)
            self.app_list_widget2.move(self.app_list_widget.x() + self.app_list_widget.width()-10,self.titel_label.height())
        itemwidget.show()
        itemwidget2.show()
        self.app_num = self.app_num +1

    #
    #函数：处理点击卡片
    #
    def slot_click_card(self,app_msg):
        self.mw.save_click_hot_card(app_msg.app_id)
        self.click_card.emit(app_msg,True,ClickCard.HOT)

    #
    # 函数：点击向右的箭头
    #
    def slot_click_right_btn(self):
        x = self.app_list_widget.x()
        x2 = self.app_list_widget2.x()
        if (self.final_x_position == 0 and self.final_x_position2 == 0):
            self.final_x_position = x - (self.card_width + 20)*self.num
            self.final_x_position2 = x2 - (self.card_width + 20)*self.num
            self.ad_move_left_time.start(1)
        else:
            self.final_x_position = self.final_x_position - (self.card_width + 20)*self.num
            self.final_x_position2 = self.final_x_position2 - (self.card_width + 20)*self.num
            self.ad_move_left_time.start(1)
        self.right_btn.setEnabled(False)
        self.left_btn.setEnabled(False)
    #
    # 函数：向左滑动
    #
    def slot_slide_left(self):
        if (self.app_list_widget.x() <= 0 ):
            sub = self.app_list_widget2.x() - self.final_x_position2
            self.app_list_widget2.move(self.app_list_widget.x() + self.app_list_widget.width() + 10, self.titel_label.height())
            self.final_x_position2 = self.app_list_widget2.x() - sub
        if (self.app_list_widget2.x() <= 0):
            sub = self.app_list_widget.x() - self.final_x_position
            self.app_list_widget.move(self.app_list_widget2.x() + self.app_list_widget2.width() + 10, self.titel_label.height())
            self.final_x_position = self.app_list_widget.x() - sub
        x = int(self.app_list_widget.x())
        x2 = int(self.app_list_widget2.x())
        if (x > self.final_x_position):
            self.ad_move_left_time.stop()
            if (x - 40 <= self.final_x_position):
                self.app_list_widget.move(self.final_x_position, self.titel_label.height())
            else:
                self.app_list_widget.move(x - 40, self.titel_label.height())
            if (x2 - 40 <= self.final_x_position2):
                self.app_list_widget2.move(self.final_x_position2, self.titel_label.height())
            else:
                self.app_list_widget2.move(x2 - 40, self.titel_label.height())
            self.app_list_widget.show()
            self.app_list_widget2.show()
            self.ad_move_left_time.start(1)
            self.right_btn.setEnabled(False)
            self.left_btn.setEnabled(False)
        else:
            self.ad_move_left_time.stop()
            self.app_list_widget.move(self.final_x_position, self.titel_label.height())
            self.app_list_widget2.move(self.final_x_position2, self.titel_label.height())
            if(Globals.NEED_SEND_HOT_EXPOSURE == True):
                if (self.app_list_widget.x() < self.width() or (self.app_list_widget.x() + self.app_list_widget.width() < self.width())):
                    for card in self.app_list_widget.children():
                        if (card.x() + self.app_list_widget.x() <= self.width() and card.x() + self.app_list_widget.x() >= 0):
                            if (card.appmsg.app_id not in self.show_card_list):
                                self.show_card_list.append(card.appmsg.app_id)
                                self.mw.save_hot_exposure(card.appmsg.app_id)
                            else:
                                b = [val for val in self.add_card_list if val not in self.show_card_list]
                                if (b == []):
                                    Globals.NEED_SEND_HOT_EXPOSURE = False
                                    self.show_card_list.clear()
                if (self.app_list_widget2.x() < self.width() or (self.app_list_widget2.x() + self.app_list_widget2.width() < self.width())):
                    for card in self.app_list_widget2.children():
                        if (card.x() + self.app_list_widget2.x() <= self.width() and card.x() + self.app_list_widget2.x() >= 0):
                            if (card.appmsg.app_id not in self.show_card_list):
                                self.show_card_list.append(card.appmsg.app_id)
                                self.mw.save_hot_exposure(card.appmsg.app_id)
                            else:
                                b = [val for val in self.add_card_list if val not in self.show_card_list]
                                if (b == []):
                                    Globals.NEED_SEND_HOT_EXPOSURE = False
                                    self.show_card_list.clear()
            else:
                self.show_card_list.clear()
            self.final_x_position = 0
            self.final_x_position2 = 0
            self.app_list_widget.show()
            self.app_list_widget2.show()
            self.right_btn.setEnabled(True)
            self.left_btn.setEnabled(True)

    #
    # 函数：点击向左的箭头
    #
    def slot_click_left_btn(self):
        x = self.app_list_widget.x()
        x2 = self.app_list_widget2.x()
        if (self.final_x_position == 0 and self.final_x_position2 == 0):
            self.final_x_position = x + (self.card_width + 20)*self.num
            self.final_x_position2 = x2 + (self.card_width + 20)*self.num
            self.ad_move_right_time.start(1)
        else:
            self.ad_move_right_time.stop()
            self.final_x_position = self.final_x_position + (self.card_width + 20)*self.num
            self.final_x_position2 = self.final_x_position2 + (self.card_width + 20)*self.num
            self.ad_move_right_time.start(1)
        self.right_btn.setEnabled(False)
        self.left_btn.setEnabled(False)

    #
    # 函数向右滑动
    #
    def slot_slide_right(self):
        if (self.app_list_widget.x() >= 0):
            sub = self.app_list_widget2.x() - self.final_x_position2
            self.app_list_widget2.move(self.app_list_widget.x() - self.app_list_widget2.width() - 20+10, self.titel_label.height())
            self.final_x_position2 = self.app_list_widget2.x() - sub
        if (self.app_list_widget2.x() >= 0):
            sub = self.app_list_widget.x() - self.final_x_position
            self.app_list_widget.move(self.app_list_widget2.x() - self.app_list_widget.width() - 20+10, self.titel_label.height())
            self.final_x_position = self.app_list_widget.x() - sub
        x = int(self.app_list_widget.x())
        x2 = int(self.app_list_widget2.x())
        if (x < self.final_x_position):
            self.ad_move_right_time.stop()
            if (x + 40 >= self.final_x_position):
                self.app_list_widget.move(self.final_x_position, self.titel_label.height())
            else:
                self.app_list_widget.move(x + 40, self.titel_label.height())
            if (x2 + 40 >= self.final_x_position2):
                self.app_list_widget2.move(self.final_x_position2, self.titel_label.height())
            else:
                self.app_list_widget2.move(x2 + 40, self.titel_label.height())
            self.app_list_widget.show()
            self.app_list_widget2.show()
            self.ad_move_right_time.start(1)
            self.right_btn.setEnabled(False)
            self.left_btn.setEnabled(False)
        else:
            self.ad_move_right_time.stop()
            self.app_list_widget.move(self.final_x_position, self.titel_label.height())
            self.app_list_widget2.move(self.final_x_position2, self.titel_label.height())
            if(Globals.NEED_SEND_HOT_EXPOSURE == True):
                if (self.app_list_widget.x() < self.width() or (self.app_list_widget.x() + self.app_list_widget.width() < self.width())):
                    for card in self.app_list_widget.children():
                        if (card.x() + self.app_list_widget.x() <= self.width() and card.x() + self.app_list_widget.x() >= 0):
                            if (card.appmsg.app_id not in self.show_card_list):
                                self.show_card_list.append(card.appmsg.app_id)
                                self.mw.save_hot_exposure(card.appmsg.app_id)
                            else:
                                b = [val for val in self.add_card_list if val not in self.show_card_list]
                                if (b == []):
                                    Globals.NEED_SEND_HOT_EXPOSURE = False
                                    self.show_card_list.clear()
                if (self.app_list_widget2.x() < self.width() or (self.app_list_widget2.x() + self.app_list_widget2.width() < self.width())):
                    for card in self.app_list_widget2.children():
                        if (card.x() + self.app_list_widget2.x() <= self.width() and card.x() + self.app_list_widget2.x() >= 0):
                            if (card.appmsg.app_id not in self.show_card_list):
                                self.show_card_list.append(card.appmsg.app_id)
                                self.mw.save_hot_exposure(card.appmsg.app_id)
                            else:
                                b = [val for val in self.add_card_list if val not in self.show_card_list]
                                if (b == []):
                                    Globals.NEED_SEND_HOT_EXPOSURE = False
                                    self.show_card_list.clear()
            else:
                self.show_card_list.clear()
            self.final_x_position = 0
            self.final_x_position2 = 0
            self.app_list_widget.show()
            self.app_list_widget2.show()
            self.right_btn.setEnabled(True)
            self.left_btn.setEnabled(True)


    # 鼠标进入事件

    def enterEvent(self, event):
        self.status = BUTTON_ENTER
        if(self.app_list_widget.width() > self.width()):
            self.right_btn.show()
            self.left_btn.show()

    #
    # 函数：鼠标离开事件
    #

    def leaveEvent(self, event):
        self.status = BUTTON_LEAVE
        self.right_btn.hide()
        self.left_btn.hide()

    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            if (hasattr(self.mw, "title_bar")):
                self.mw.title_bar.exit_login_btn.hide()
                self.mw.title_bar.senior_search.hide()
            if (hasattr(self.mw, "search_tip_widget")):
                self.mw.search_tip_widget.hide()
            self.status = BUTTON_PRESSED
        else:
            event.ignore()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import sqlite3
import datetime
import threading
from models.enums import UKSC_CACHE_DIR
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
lock = threading.Lock()
_ = gettext.gettext

DB_FILE = UKSC_CACHE_DIR+"/history_installation.db"

class Record_History_Installation:
    def __init__(self):
        if os.path.isfile(DB_FILE):
            try:
                self.connect = sqlite3.connect(DB_FILE, check_same_thread=False)
                self.cursor = self.connect.cursor()
                select_table = ".table"
                try:
                    lock.acquire(True)
                    sql = "select name from sqlite_master where type = 'table' and name = 'installed'"
                    self.cursor.execute(sql)
                    res = self.cursor.fetchall()
                    if(res == None or res == []):
                        self.cursor.execute('''create table installed(appname text)''')
                        self.connect.commit()
                except Exception as e:
                        pass
                finally:
                    lock.release()
            except:
                pass
        else:
            try:
                self.connect = sqlite3.connect(DB_FILE, check_same_thread=False)
                self.cursor = self.connect.cursor()
                self.cursor.execute('''create table installed(appname text)''')
                self.connect.commit()
                self.cursor.close()
                self.connect.close()
                self.connect = sqlite3.connect(DB_FILE, check_same_thread=False)
                self.cursor = self.connect.cursor()
                os.system("chmod 664 " + DB_FILE)
            except:
                pass

    #
    # 函数:将软件的安装信息写入数据库
    #
    def insert(self, app_name):
        in_db = False
        self.select()
        res = self.cursor.fetchall()
        for i in res:
            if(app_name == i[0]):
                in_db = True
        if(in_db == False):
            try:
                self.cursor.execute("insert into installed values(?)",(app_name,))
            except:
                pass
        self.connect.commit()
        self.cursor.close()
        self.connect.close()

    #
    # 函数:读取数据库的信息
    #
    def select(self):
        try:
            self.cursor.execute("select appname from installed")
        except:
            pass




#!/usr/bin/python3
# -*- coding: utf-8 -*-


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from models.enums import BUTTON_DISABLE,BUTTON_PRESSED,BUTTON_LEAVE,BUTTON_ENTER,AppMsg
from utils import commontools
from models.application import *

class Popular_Card_Widget(QFrame):
    clicked_card = pyqtSignal(Application)
    def __init__(self, app_msg, card_width ,card_height,card_path,parent=None):
        QWidget.__init__(self, parent)
        self.m_parent = parent
        self.card_width = card_width
        self.card_height = card_height
        self.appmsg = app_msg
        self.card_path = card_path
        self.init_ui()

    def init_ui(self):
        self.resize(self.card_width, self.card_height)
        self.setFocusPolicy(Qt.NoFocus)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

        self.card_label = QLabel(self)
        self.card_label.resize(288,190)
        self.card_label.move(0, 0)
        cover_image_name = self.card_path.split("/")[len(self.card_path.split("/"))-1]
        iconpath = commontools.get_coverimage(cover_image_name)

        if (Globals.DEVICE == 1):
            self.card_label.setStyleSheet("QLabel{border-radius: 12px;}")
            pix2 = QPixmap(iconpath)
            fitpixmap_userIcon = pix2.scaled(self.card_label.width(), self.card_label.height(), Qt.IgnoreAspectRatio,
                                             Qt.SmoothTransformation)
            fitpixmap_userIcon = self.PixmapToRound(fitpixmap_userIcon, self.width(), self.height())
            self.card_label.setPixmap(fitpixmap_userIcon)
        else:
            self.card_label.setStyleSheet("QLabel{border-image:url('" + iconpath + "');border-radius: 12px;}")

        self.hover_widget = QWidget(self)
        self.hover_widget.resize(self.width(), self.height())
        self.hover_widget.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        # self.card_list_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.hover_widget.setStyleSheet("QWidget{background: rgba(0, 0, 0, 0.1);border-radius: 12px;}")
        self.hover_widget.move(0, 0)
        self.hover_widget.hide()


    def PixmapToRound(self,src,x,y):
        pixmapa = QPixmap(src)
        pixmap = QPixmap(x, y)
        pixmap.fill(Qt.transparent)
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.Antialiasing, True)
        # painter.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        path = QPainterPath()
        rect = QRectF(0,0,x,y)
        path.addRoundedRect(rect, 12.0, 12.0)
        painter.setClipPath(path)
        painter.drawPixmap(0, 0, x, y, pixmapa)

        return pixmap


    # 鼠标进入事件
    def enterEvent(self, event):
        self.status = BUTTON_ENTER
        self.setGeometry(self.x(), self.y(), self.width() , self.height())
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setOffset(0, 0)
        self.shadow.setColor(QColor(qRgba(149,149,149,0)))
        self.shadow.setBlurRadius(12)
        self.setGraphicsEffect(self.shadow)
        self.hover_widget.show()

    #
    # 函数：鼠标离开事件
    #
    def leaveEvent(self, event):
        self.status = BUTTON_LEAVE
        self.resize(self.width(),self.height())
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(0)
        self.shadow.setOffset(0, 0)
        self.setGraphicsEffect(self.shadow)
        self.hover_widget.hide()

    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.status = BUTTON_PRESSED
            self.clicked_card.emit(self.appmsg)
        else:
            event.ignore()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE

# Copyright (C) 2013 National University of Defense Technology(NUDT) & Kylin Ltd

# Author:
#     Shine Huang<shenghuang@ubuntukylin.com>
# Maintainer:
#     Shine Huang<shenghuang@ubuntukylin.com>

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import dbus
import dbus.service
import os
import dbus.mainloop.glib
import getpass
import signal
from gi.repository import GObject
from gi.repository import GLib

INTERFACE = 'com.kylin.utiliface'
DBUSPATH = '/'

class UtilIfaceDbusService(dbus.service.Object):

    def __init__(self, bus, mainloop):

        self.bus = bus
        self.bus_name = dbus.service.BusName(INTERFACE, bus=bus)

        dbus.service.Object.__init__(self, self.bus_name, DBUSPATH)
        self.mainloop = mainloop

    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
    def exit(self):
        self.mainloop.quit()

    # 获取安全配置文件存放路径
    @dbus.service.method('com.kylin.utiliface')
    def GetSecurityConfigPath(self):
        SOFTWARE_BLACK="/home/"+getpass.getuser()+"/.config/kylin-software-center-security-config.json"
        return SOFTWARE_BLACK

    # 重新加载安全配置（软件商店只提供接口，动态重载后续开发）
    @dbus.service.method('com.kylin.utiliface')
    def ReloadSecurityConfig(self):
        pass
    


if __name__ == '__main__':
    os.environ["TERM"] = "xterm"
    os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X11R6/bin"
    os.environ["DEBIAN_FRONTEND"] = "noninteractive"

    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    # GObject.threads_init()
    # mainloop = GObject.MainLoop()
    mainloop = GLib.MainLoop()
    signal.signal(signal.SIGINT, lambda: mainloop.quit())
    UtilIfaceDbusService(dbus.SessionBus(), mainloop)
    mainloop.run()



 #!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
import glob
from setuptools import setup
import DistUtilsExtra.command.build_extra
import DistUtilsExtra.command.build_i18n
import DistUtilsExtra.command.clean_i18n
from subprocess import call
#from DistUtilsExtra.command import build_extra
#import build_i18n_ext as build_i18n


PO_DIR = 'po'

for po in glob.glob(os.path.join(PO_DIR, '*.po')):
    lang = os.path.basename(po[:-3])
    mo = os.path.join(PO_DIR, 'kylin-software-center.mo')
    target_dir = os.path.dirname(mo)
    if not os.path.isdir(target_dir):
        os.makedirs(target_dir)
    try:
        return_code = call(['msgfmt', '-o', mo, po])
    except OSError:
        print('Translation not available, please install gettext')
        break
    if return_code:
        raise Warning('Error when building locales')
cmdclass ={
            "build" : DistUtilsExtra.command.build_extra.build_extra,
            "build_i18n" :  DistUtilsExtra.command.build_i18n.build_i18n,
            #"clean": DistUtilsExtra.command.clean_i18n.clean_i18n,
}


data_files=[
    ('bin/', ['ubuntu-kylin-software-center']),
    ('../etc/dbus-1/system.d/', ['backend/aptdaemon/conf/com.kylin.softwarecenter.conf']),
    ('share/dbus-1/system-services/', ['backend/aptdaemon/conf/com.kylin.softwarecenter.service']),
    ('share/polkit-1/actions/', ['backend/aptdaemon/conf/com.kylin.softwarecenter.policy']),
    ('lib/python3/dist-packages/kylin-software-center-daemon/', glob.glob('backend/aptdaemon/dbus_service')),
    ('share/applications/',['kylin-software-center.desktop']),
    ('share/pixmaps/',['kylin-software-center.png']),
    ('../etc/rsyslog.d/',['58-kylin-software-center.conf']),
    ('share/icons/hicolor/128x128/apps/',['kylin-software-center.png']),
    ('share/kylin-software-center/backend', glob.glob('backend/*.py')),
    ('share/kylin-software-center/backend/service/', glob.glob('backend/service/*.txt')),
    ('share/kylin-software-center/backend/run_app/', glob.glob('backend/run_app/*.py')),
    ('share/kylin-software-center/backend/service/', glob.glob('backend/service/*.py')),
    ('share/kylin-software-center/backend/service/', glob.glob('backend/service/*.txt')),
    ('share/kylin-software-center/data/ads/', glob.glob('data/ads/*.png')),
    ('share/kylin-software-center/', ['kylin-software-center.conf']),
    ('share/kylin-software-center/data/icons/', glob.glob('data/icons/*.png')),
    ('share/kylin-software-center/data/coverimage/', glob.glob('data/coverimage/*')),
    ('share/kylin-software-center/data/new_arrivals/', glob.glob('data/new_arrivals/*')),
    ('share/kylin-software-center/data/user_profile/', glob.glob('data/user_profile/*')),
    ('share/kylin-software-center/data/clientAPItest/', glob.glob('data/clientAPItest/*')),
    ('share/kylin-software-center/data/winicons/', glob.glob('data/winicons/*')),
    ('share/kylin-software-center/data/', ['data/uksc.db']),
    ('share/kylin-software-center/data/clientAPItest/', glob.glob('data/clientAPItest/*')),
    ('share/kylin-software-center/models/', glob.glob('models/*')),
    ('share/kylin-software-center/res/', glob.glob('res/*.png')),
    ('share/kylin-software-center/res/', glob.glob('res/*.gif')),
    ('share/kylin-software-center/res/uninstall/', glob.glob('res/uninstall/*')),
    ('share/kylin-software-center/ui/', glob.glob('ui/*')),
    ('share/kylin-software-center/utils/', glob.glob('utils/*')),
    ('share/kylin-software-center/po/', glob.glob('po/*')),
    ('share/kylin-software-center/kydroid/', glob.glob('kydroid/*')),
    ('share/kylin-software-center/service/', glob.glob('service/*')),
    ('share/kylin-software-center/',['kylin-software-center.py']),
    ('share/kylin-user-guide/data/guide/kylin-store/zh_CN/image/',glob.glob('kylin-software-center-help/zh_CN/image/*')),
    ('share/kylin-user-guide/data/guide/kylin-store/',['kylin-software-center-help/software-store.png']),
    ('share/kylin-user-guide/data/guide/kylin-store/zh_CN/',['kylin-software-center-help/zh_CN/index.md']),
    ('share/kylin-user-guide/data/guide/kylin-store/en_US/image/',glob.glob('kylin-software-center-help/en_US/image/*')),
    ('share/kylin-user-guide/data/guide/kylin-store/en_US/',['kylin-software-center-help/en_US/index.md']),
    # ('../etc/xdg/autostart/',['kylin-software-center-autostart.desktop']),
    ('lib/systemd/user/',['kylin-software-center.service']),
    ('/lib/systemd/system/',['backend/aptdaemon/conf/softwarecenter-daemon.service']),
    ('share/dbus-1/services/', ['backend/utildaemon/conf/com.kylin.utiliface.service']),
    ]

#
# 函数名:找.mo文件
# Function: find .mo file
# 
def find_mo_files():
    data_files = []
    for mo in glob.glob(os.path.join(PO_DIR, '*', 'kylin-software-center.mo')):
        lang = os.path.basename(os.path.dirname(mo))
        dest = os.path.join('share', 'locale', lang, 'LC_MESSAGES')
        data_files.append((dest, [mo]))
    return data_files

data_files.extend(find_mo_files())



setup(name="kylin-software-center",
    version="1.3.10",
    author="Ubuntu Kylin Team",
    author_email="ubuntukylin-members@list.launchpad.net",
    url="https://launchpad.net/kylin-software-center",
    license="GNU General Public License (GPL)",
    packages = [ 'kylin_software_center_daemon',],
    package_dir = {
        '': '.',
    },
    install_requires = [ 'setuptools', ],
    cmdclass = cmdclass,
    data_files=data_files,
)




#!/usr/bin/python3
# -*- coding: utf-8 -*-


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from models.enums import BUTTON_ENTER,BUTTON_LEAVE,BUTTON_PRESSED,BUTTON_DISABLE
from ui.popular_app_card import *
from ui.systembutton import *
from ui.category_widget import *
from ui.ad_widget import *
from models.application import *
from models.apkinfo import *
from models.enums import setLongTextToElideFormat
import os
import sip
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

class Drive_Widget(QFrame):
    clicked_btn = True
    is_first_click = True
    click_type_btn = pyqtSignal(str,bool)
    click_category_ad = pyqtSignal(Application)
    click_category_ad_apk = pyqtSignal(ApkInfo)
    click_none_card = pyqtSignal()
    card_width = 248
    card_height = 124
    card_space = 15
    widget_space = 50
    def __init__(self,w,h,parent = None):
        QFrame.__init__(self,parent)
        self.w = w
        self.h = h
        self.m_parent = parent
        self.init_ui()
        self.init_style()

    def init_ui(self):
        self.resize(self.w, self.h)
        self.setWindowFlags(Qt.FramelessWindowHint)
        #self.setAttribute(Qt.WA_TranslucentBackground)

        self.btn_widget = CategoryBar(Globals.DRIVE_CATEGORIES,self,False)
        self.btn_widget.setAttribute(Qt.WA_TranslucentBackground)
        # self.btn_widget.resize((self.btn_widget.visiblecategorycount - 1) * (
        #         self.btn_widget.itemwidth + self.btn_widget.categoryspacing) + self.btn_widget.itemwidth, 64)
        self.btn_widget.move(int((self.w - self.btn_widget.width()) / 2), 20)
        self.btn_widget.show()

        self.type_widget = QWidget(self)
        # self.type_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.type_widget.resize(self.w,self.h-self.btn_widget.height())
        self.type_widget.move(0,self.btn_widget.height())
        self.type_widget.setStyleSheet("QWidget{background-color:transparent;border-radius:0px;}")
        self.type_widget.show()

        self.type_name_label = QLabel(self.type_widget)
        self.type_name_label.setAttribute(Qt.WA_TranslucentBackground)
        self.type_name_label.setFixedHeight(42)
        self.type_name_label.setStyleSheet("QLabel{font-size: 30px;font-weight: 600;color: #303133;line-height: 42px;}")
        self.type_name_label.setText(self.btn_widget.first_btn)
        self.type_name_label.adjustSize()
        self.type_name_label.move(self.widget_space,0)
        self.type_name_label.show()
        # self.type_name_label.hide()
        self.line_label = QLabel(self.type_widget)
        self.line_label.setFixedSize(self.width()-self.widget_space*2,1)
        self.line_label.setStyleSheet("QLabel{border: 1px solid #E5E7EE;background-color:#E5E7EE;}")
        # self.line_label.move(self.widget_space,0)
        # self.line_label.move(self.widget_space,self.type_name_label.y()+self.type_name_label.height()+10)

        self.type_show_card_widget = QWidget(self.type_widget)
        # self.type_show_card_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.type_show_card_widget.resize(self.type_widget.width() - self.widget_space - self.widget_space,
                                          self.type_widget.height() - self.type_name_label.height())
        self.type_show_card_widget.move(self.widget_space, self.line_label.y() + self.line_label.height()+10)
        self.type_show_card_widget.setStyleSheet("QWidget{background-color:transparent;border-radius:0px;}")
        self.type_show_card_widget.show()
        self.type_show_card_widget.children_count = 0

        self.one_click_download = QPushButton(self.type_widget)
        self.one_click_download.setFocusPolicy(Qt.NoFocus)
        self.one_click_download.setText(_("all down"))
        self.one_click_download.adjustSize()
        self.one_click_download.setFixedSize(80, 26)
        self.one_click_download.setStyleSheet(
            "QPushButton{border:0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;border:0px solid #ff0000;}QPushButton:hover{background: rgba(55, self.widget_space, 250, 0.12);border-radius: 12px;}QPushButton:pressed{background: rgba(55, self.widget_space, 250, 0.12);border-radius: 12px;}")
        self.one_click_download.move(self.type_show_card_widget.width() - self.one_click_download.width() + self.type_show_card_widget.x() - 10,
                                     self.type_name_label.y())
        self.one_click_download.setEnabled(False)

        self.all_check_btn = QPushButton(self.type_widget)
        self.all_check_btn.setFocusPolicy(Qt.NoFocus)
        self.all_check_btn.setText(_("Select all"))
        self.all_check_btn.setFixedSize(80, 26)
        self.all_check_btn.setStyleSheet(
            "QPushButton{font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;background: transparent;border-radius: 13px;}QPushButton::hover{color: #3790FA;background: transparent;font-weight: 400;}QPushButton:pressed{color: #3790FA;background: rgba(55, self.widget_space, 250, 0.12);font-weight: 500;}")
        self.all_check_btn.move(self.one_click_download.x() - self.all_check_btn.width() + 5, self.type_name_label.y())

        self.loadingmove = QMovie("./res/loadgif2.gif")
        self.loadingmove_dark = QMovie("./res/loadgifdark.gif")
        self.loadinglabel = QLabel(self)
        self.loadinglabel.setAttribute(Qt.WA_TranslucentBackground)
        self.loadinglabel.resize(2000,2000)
        self.loadinglabel.move(int((self.width()-self.loadinglabel.width())/2),int((self.height()-self.loadinglabel.height())/2))
        self.loadinglabel.setAlignment(Qt.AlignCenter)
        if(Globals.THEME_COLOR == 0):
            self.loadinglabel.setMovie(self.loadingmove)
            self.loadingmove.start()
        else:
            self.loadinglabel.setMovie(self.loadingmove_dark)
            self.loadingmove_dark.start()


        self.scrollArea = QScrollArea(self)
        self.scrollArea.setWidget(self.type_widget)
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollArea.horizontalScrollBar().setEnabled(False)
        self.scrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scrollArea.setGeometry(0, int(self.btn_widget.height()+self.btn_widget.y()+16), self.type_widget.width()-10, self.height()-self.btn_widget.height()-10)
        self.type_widget.resize(self.width(),int(self.type_name_label.y()+self.type_name_label.height()+self.type_show_card_widget.height()))
        self.scrollArea.show()
        self.re_move()

    #
    #函数：初始化界面的样式
    #
    def init_style(self):
        if(Globals.THEME_COLOR == 0):
            self.setStyleSheet("QFrame{border-bottom-left-radius:12px;border-bottom-right-radius:12px; background-color:#ffffff;}")
            self.btn_widget.setStyleSheet("QWidget{background-color:#ffffff;border-radius:0px;}")
            palette1 = QPalette()
            palette1.setColor(self.type_widget.backgroundRole(), QColor(255, 255, 255))  # 设置背景颜色
            self.type_widget.setPalette(palette1)
            self.type_widget.setAutoFillBackground(True)
            # self.type_widget.setStyleSheet("QWidget{background-color:#ffffff;border-radius:0px;}")
            # self.type_show_card_widget.setStyleSheet("QWidget{background-color:#ffffff;border-radius:0px;}")
            self.type_name_label.setStyleSheet("QLabel{font-size: 24px;font-weight: 500;color: #303133;line-height: 42px;}")
            self.line_label.setStyleSheet("QLabel{border: 1px solid #E5E7EE;background-color:#E5E7EE;}")
            self.loadinglabel.setMovie(self.loadingmove)
            self.loadingmove.start()
            if (self.one_click_download.isEnabled()):
                self.one_click_download.setStyleSheet(
                    "QPushButton{border:0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;border:0px solid #ff0000;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}QPushButton:pressed{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}")
            else:
                self.one_click_download.setStyleSheet(
                    "QPushButton{font-size:14px;font-weight: 500;color: #8f9399;line-height: 20px;border:0px solid #ff0000;}")
            self.all_check_btn.setStyleSheet(
                "QPushButton{font-size: 14px;font-weight: 400;line-height: 20px;border:0px;font-weight: 500;color: #606265;line-height: 20px;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}PushButton:pressed{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}")
            self.scrollArea.verticalScrollBar().setStyleSheet("QScrollBar:vertical{margin:0px 0px 0px 0px;background-color:transparent;border:0px;width:6px;border-radius: 3px;}\
                                QScrollBar::handle:vertical{background-color:#C2C2C2;width:6px;border-radius:3px;} QScrollBar::handle:vertical:hover{background-color:#8F9399;opacity:0.1;width:6px;border-radius:3px;}  QScrollBar::handle:vertical:pressed{background-color:#8F9399;width:6px;border-radius:3px;}\
                                QScrollBar::sub-line:vertical{subcontrol-origin:margin;border:0px solid red;height:13px}\
                                QScrollBar::up-arrow:vertical{subcontrol-origin:margin;background-color:rgba(0,0,0,0);height:13px}\
                                QScrollBar::sub-page:vertical{background-color:transparent;}\
                                QScrollBar::add-page:vertical{background-color:transparent;}\
                                QScrollBar::down-arrow:vertical{background-color:rgba(0,0,0,0);}\
                                QScrollBar::add-line:vertical{subcontrol-origin:margin;border:0px solid green;height:13px}")
            for btn in self.btn_widget.btnGroup.buttons():
                if(self.type_name_label.text() == btn.text_label.text()):
                    btn.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;text-align: center;background-color: #3790FA;color: #ffffff;text-align : center;}")
                else:
                    btn.setStyleSheet(
                        "QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #606265;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")
        else:
            self.setStyleSheet("QFrame{border-bottom-left-radius:12px;border-bottom-right-radius:12px; background-color:#1F2022;}")
            self.btn_widget.setStyleSheet("QWidget{background-color:#1F2022;border-radius:0px;}")
            palette1 = QPalette()
            palette1.setColor(self.type_widget.backgroundRole(), QColor(31, 32, 34))  # 设置背景颜色
            self.type_widget.setPalette(palette1)
            self.type_widget.setAutoFillBackground(True)
            # self.type_widget.setStyleSheet("QWidget{background-color:#1F2022;border-radius:0px;}")
            # self.type_show_card_widget.setStyleSheet("QWidget{background-color:#1F2022;border-radius:0px;}")
            self.type_name_label.setStyleSheet("QLabel{font-size: 24px;font-weight: 500;color: #F9F9F9;line-height: 42px;}")
            self.line_label.setStyleSheet("QLabel{border: 1px solid #393939;background-color:#393939;}")
            self.loadinglabel.setMovie(self.loadingmove_dark)
            self.loadingmove_dark.start()
            if (self.one_click_download.isEnabled()):
                self.one_click_download.setStyleSheet(
                    "QPushButton{border:0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;border:0px solid #ff0000;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}QPushButton:pressed{background: rgba(55, 144, 250, 0.12);border-radius: 12px;}")
            else:
                self.one_click_download.setStyleSheet(
                    "QPushButton{font-size:14px;font-weight: 500;color: #8f9399;line-height: 20px;border:0px solid #ff0000;}")
            self.all_check_btn.setStyleSheet("QPushButton{font-size: 14px;border:0px;font-weight: 500;color: #F9F9F9;line-height: 20px;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}PushButton:pressed{background: rgba(55, 144, 250, 0.12);color: #3790FA;border-radius: 12px;}")
            self.scrollArea.verticalScrollBar().setStyleSheet("QScrollBar:vertical{margin:0px 0px 0px 0px;background-color:transparent;border:0px;width:6px;border-radius: 3px;}\
                                QScrollBar::handle:vertical{background-color:#6B6B6B;width:6px;border-radius:3px;} QScrollBar::handle:vertical:hover{background-color:#959595;opacity:0.1;width:6px;border-radius:3px;}  QScrollBar::handle:vertical:pressed{background-color:#959595;width:6px;border-radius:3px;}\
                                QScrollBar::sub-line:vertical{subcontrol-origin:margin;border:0px solid red;height:13px}\
                                QScrollBar::up-arrow:vertical{subcontrol-origin:margin;background-color:rgba(0,0,0,0);height:13px}\
                                QScrollBar::sub-page:vertical{background-color:transparent;}\
                                QScrollBar::add-page:vertical{background-color:transparent;}\
                                QScrollBar::down-arrow:vertical{background-color:rgba(0,0,0,0);}\
                                QScrollBar::add-line:vertical{subcontrol-origin:margin;border:0px solid green;height:13px}")
            for btn in self.btn_widget.btnGroup.buttons():
                if(self.type_name_label.text() == btn.text_label.text()):
                    btn.setStyleSheet("QPushButton{border-radius: 12px;font-size: 14px;font-weight: 400;line-height: 20px;text-align: center;background-color: #3790FA;color: #ffffff;text-align : center;}")
                else:
                    btn.setStyleSheet(
                        "QPushButton{background-color:transparent;border-radius: 12px;font-size: 14px;font-weight: 400;color: #8F9399;text-align: center;line-height: 20px;text-align : center;}QPushButton:hover{background: rgba(55, 144, 250, 0.12);color: #3790FA;}QPushButton:pressed{background-color: #3790FA;color: #ffffff;}")

    #
    #函数：重新设置界面的控件位置和大小
    #
    def re_move(self):
        # self.btn_widget.resize((self.btn_widget.visiblecategorycount - 1) * (
        #         self.btn_widget.itemwidth + self.btn_widget.categoryspacing) + self.btn_widget.itemwidth, 64)
        self.btn_widget.reload_btn()
        self.btn_widget.move(int((self.width() - self.btn_widget.width()) / 2), 20)
        self.type_widget.resize(self.width(), self.height() - self.btn_widget.height()-self.btn_widget.y())
        self.type_name_label.move(self.widget_space, 0)
        self.type_widget.resize(self.type_widget.width(),0)
        self.type_show_card_widget.resize(self.type_widget.width() - self.widget_space - self.widget_space,self.type_widget.height() - self.type_name_label.height())
        self.loadinglabel.move(int((self.width() - self.loadinglabel.width()) / 2),int((self.height() - self.loadinglabel.height()) / 2))
        self.line_label.setFixedSize(self.width() - self.widget_space * 2, 1)
        self.line_label.move(self.widget_space, self.type_name_label.height()+10)
        self.type_show_card_widget.move(self.widget_space, self.line_label.y() + self.line_label.height()+10)
        self.one_click_download.move(
            self.type_show_card_widget.width() - self.one_click_download.width() + self.type_show_card_widget.x() - 10,
            self.type_name_label.y())
        self.all_check_btn.move(self.one_click_download.x() - self.all_check_btn.width(), self.type_name_label.y())
        self.scrollArea.setGeometry(0, int(self.btn_widget.height() + self.btn_widget.y()) + 16,
                                    self.type_widget.width() - 10,
                                    int(self.height() - self.btn_widget.height() - self.btn_widget.y() - 30))
        self.type_show_card_widget.move(self.widget_space, int(self.type_name_label.y() + self.type_name_label.height()))
        self.reload_card(self.type_show_card_widget, self.scrollArea)
        self.type_widget.resize(self.type_widget.width(),
                                int(self.type_name_label.y() + self.type_name_label.height() + self.type_show_card_widget.height()))
        # self.scrollArea.setGeometry(0, self.btn_widget.height()+self.btn_widget.y()+16, self.type_widget.width() - 10, self.height() - self.btn_widget.height() - 10)
        # self.reload_card(self.type_show_card_widget,self.scrollArea)
        # self.type_widget.resize(self.type_widget.width(), self.type_show_card_widget.height()+self.line_label.y()+self.line_label.height())
        # self.type_widget.move(0, self.btn_widget.height() + self.btn_widget.y())


    #
    #函数：重新设置卡片
    #
    def reload_card(self,widget,scrollArea):
        cards = widget.children()
        if (len(cards) != 0):
            h_num = int(widget.width()/(self.card_width+self.card_space))
            get_card_width = int((widget.width() - (self.card_space * (h_num - 1)) - 20) / h_num)
            proportion = get_card_width / widget.width()
            get_card_height = self.card_height
            v_num = 0
            num = 0
            for i in range(len(cards)):
                card = cards[i]
                card.resize(get_card_width,get_card_height)
                card.re_move()
                if (num < h_num):
                    if (num == 0):
                        card.move(0, 20 + (20 + card.height()) * v_num)
                        widget.resize(widget.width(), 20 + (20 + card.height()) * (v_num + 1))
                    else:
                        card.move(num * (card.width() + self.card_space) + 5, 20 + (20 + card.height()) * v_num)
                    num = num + 1
                else:
                    num = 0
                    v_num = v_num + 1
                    if (num == 0):
                        card.move(0, 20+ (20 + card.height()) * v_num)
                        widget.resize(widget.width(),  (20 + card.height()) * (v_num + 1))
                    else:
                        card.move(num * (card.width() + self.card_space), 20 + (20 + card.height()) * v_num)
                    num = num + 1

    #
    #函数：处理点击分类界面的类型按钮
    #
    def slot_clicked_btn(self,btn_name):
        self.click_type_btn.emit(btn_name,True)

    # 鼠标进入事件

    def enterEvent(self, event):
        self.status = BUTTON_ENTER
        self.scrollArea.verticalScrollBar().show()
        # self.scrollArea.show()

    #
    # 函数：鼠标离开事件
    #

    def leaveEvent(self, event):
        self.status = BUTTON_LEAVE
        self.scrollArea.verticalScrollBar().hide()

    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.m_parent.mw.hide_tip_widget()
            self.status = BUTTON_PRESSED
        else:
            event.ignore()

    #
    # 函数名:设置滚动条起始位置
    # Function:set verticalSscrollBar starting point
    #
    def scrollToTop(self):
        vsb = self.scrollArea.verticalScrollBar()
        vsb.setValue(0)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from models.enums import CACHE_DIR,BUTTON_PRESSED,BUTTON_ENTER,BUTTON_LEAVE,AppMsg,PageStates,ClickCard
from models.application import *
from utils.commontools import *
import threading
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext


class New_Arrivals_Widget(QWidget):
    click_ad_card = pyqtSignal(Application,bool,int)
    start_move_time = QTimer()
    ad_move_left_time = QTimer()
    final_x_position = 0
    final_x_position2 = 0
    def __init__(self,w,mw,parent=None):
        QWidget.__init__(self, parent)
        self.appmsg = None
        self.mw = mw
        self.num = 0
        self.w = w
        self.card_w = 496
        self.card_h = 170
        self.add_card_app_id = []
        self.show_id_list = [] #保存首页发送过的软件id，为了处理一个界面没有进行界面切换时，每个应用仅保存一次
        self.init_ui()
        self.init_connect()
        # self.move_card = Move_Thread(self)

    def init_ui(self):
        self.resize(self.w,222+10)
        self.setFocusPolicy(Qt.NoFocus)
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

        self.titellabel = QLabel(self)
        self.titellabel.setFixedSize(600,26)
        self.titellabel.setAttribute(Qt.WA_TranslucentBackground)
        self.titellabel.setStyleSheet("QLabel{width: 94px;height: 33px;font-size: 24px;font-weight: 500;color: #303133;line-height: 33px;}")
        self.titellabel.setText(_("New Arrivals"))
        # self.titellabel.adjustSize()
        self.titellabel.move(10,0)

        self.more_btn = QPushButton(self)
        self.more_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.more_btn.setFocusPolicy(Qt.NoFocus)
        self.more_btn.setStyleSheet("QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;}")
        self.more_btn.setText(_("more"))
        self.more_btn.adjustSize()
        self.more_btn.move(self.width()-self.more_btn.width(),self.titellabel.y()+int((self.titellabel.height()-self.more_btn.height())/2))
        self.more_btn.show()

        self.card_list_widget = QWidget(self)
        self.card_list_widget.setFixedSize(self.width(), self.card_h+20)
        self.card_list_widget.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        #self.card_list_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.card_list_widget.setStyleSheet("QWidget{border:0px solid #000000;border-radius: 12px;background-color:transparent;}")
        self.card_list_widget.move(0, self.titellabel.height()+10)
        self.card_list_widget.show()

        self.card_list_widget2 = QWidget(self)
        self.card_list_widget2.setFixedSize(self.width(), self.card_h+20)
        self.card_list_widget2.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        #self.card_list_widget2.setAttribute(Qt.WA_TranslucentBackground)
        self.card_list_widget2.setStyleSheet("QWidget{border:0px solid #000000;border-radius: 12px;background-color:transparent;}")
        self.card_list_widget2.move(self.card_list_widget.width() , self.titellabel.height()+10)
        self.card_list_widget2.hide()

    def init_connect(self):
        # self.right_btn.clicked.connect(self.slot_click_right_btn)
        # self.left_btn.clicked.connect(self.slot_click_left_btn)
        self.ad_move_left_time.timeout.connect(self.slot_slide_left)
        self.start_move_time.timeout.connect(self.slot_click_right_btn)

    def get_lists(self, dblist, applist):
        self.dblist = []
        self.applist = []
        self.applist.extend(applist)
        self.dblist.extend(dblist)

    def add_ad_widget(self,appname, appmsg, ad_count,path):
        self.card_list_widget2.hide()
        if(appmsg != None):
            self.add_card_app_id.append(appmsg.app_id)
        itemwidget = New_APP_Card_Widget(self.card_w,self.card_h, appmsg, appname,path,self,self.card_list_widget)
        itemwidget2 = New_APP_Card_Widget(self.card_w,self.card_h, appmsg, appname,path,self,self.card_list_widget2)
        itemwidget.clicked_ad_card.connect(lambda: self.slot_click_ad_card(itemwidget.appmsg))
        itemwidget2.clicked_ad_card.connect(lambda: self.slot_click_ad_card(itemwidget2.appmsg))
        if (ad_count == 1):
            itemwidget.move(10, 10)
            itemwidget2.move(10, 10)
            itemwidget.resize(self.card_w, self.card_h)
            itemwidget2.resize(self.card_w, self.card_h)
            self.card_list_widget.move(0, self.titellabel.height()+10)
            self.card_list_widget.show()
            self.card_list_widget2.hide()
        else:
            if (ad_count == 0):
                itemwidget.move(10, 10)
                itemwidget2.move(10, 10)
                self.card_list_widget.setFixedWidth(itemwidget.width()+20)
                self.card_list_widget2.setFixedWidth(itemwidget.width()+20)
                self.card_list_widget2.move(self.card_list_widget.x() + self.card_list_widget.width(), self.titellabel.height()+10)
                self.card_list_widget.show()
            else:
                itemwidget.move((self.num) * (itemwidget.width() + 20) + 10, 10)
                itemwidget2.move(self.num * (itemwidget.width() + 20) + 10, 10)
                self.card_list_widget.setFixedWidth(self.num * (itemwidget.width() + 20) + itemwidget.width()+20)
                self.card_list_widget2.setFixedWidth(self.num * (itemwidget.width() + 20) + itemwidget.width()+20)
                self.card_list_widget2.move(self.card_list_widget.x() + self.card_list_widget.width(), self.titellabel.height()+10)
                self.card_list_widget.show()
        self.final_x_position = self.card_list_widget.x()
        self.final_x_position2 = self.card_list_widget2.x()
        itemwidget.show()
        itemwidget2.show()
        self.num = self.num + 1

    #
    #函数：添加卡片结束,显示界面
    #
    def add_card_over(self):
        if(Globals.NOWPAGE == PageStates.HOMEPAGE):
            if (self.card_list_widget.x() < self.width() or (self.card_list_widget.x() + self.card_list_widget.width() < self.width())):
                for card in self.card_list_widget.children():
                    if (card.x() + self.card_list_widget.x() <= self.width() and card.x() + self.card_list_widget.x() >= 0):
                        self.mw.save_new_arrivals_exposure(card.appmsg.app_id)
            if (self.card_list_widget2.x() < self.width() or (
                    self.card_list_widget2.x() + self.card_list_widget2.width() < self.width())):
                for card in self.card_list_widget2.children():
                    if (card.x() + self.card_list_widget2.x() <= self.width() and card.x() + self.card_list_widget2.x() >= 0):
                        self.mw.save_new_arrivals_exposure(card.appmsg.app_id)
        if (self.card_list_widget.width() > self.width()):
            self.card_list_widget.move(0, self.titellabel.height()+10)
            self.card_list_widget2.move(self.card_list_widget.x() + self.card_list_widget.width() , self.titellabel.height() + 10)
            self.start_move_time.start(4000)

    #
    # 函数：处理点击卡片
    #
    def slot_click_ad_card(self, app_msg):
        if(app_msg != None):
            self.mw.save_click_new_arrivals_card(app_msg.app_id)
            self.click_ad_card.emit(app_msg,True,ClickCard.NEWARRIVALS)
        else:
            self.mw.show_tip_widget(_("Native architecture does not support, please wait!"))
            self.mw.tip_widget.btn.clicked.connect(self.mw.slot_close_tipwidget)

    #
    # 函数：点击向右的箭头
    #
    def slot_click_right_btn(self):
        self.start_move_time.stop()
        x = self.card_list_widget.x()
        x2 = self.card_list_widget2.x()
        if (self.final_x_position == 0 and self.final_x_position2 == 0):
            self.final_x_position = x - ((self.card_w +20)*2 )
            self.final_x_position2 = x2 - ((self.card_w + 20)*2 )
            self.ad_move_left_time.start(1)
        else:
            self.final_x_position = self.final_x_position - ((self.card_w + 20)*2 )
            self.final_x_position2 = self.final_x_position2 - ((self.card_w + 20)*2 )
            self.ad_move_left_time.start(1)

    #
    # 函数：向左滑动
    #
    def slot_slide_left(self):
        if ((self.card_list_widget.x() + self.card_list_widget.width() == (self.card_w) * 2 + 40) or (self.card_list_widget.x() + self.card_list_widget.width() <= self.card_w * 2 + 40)):
            sub = self.card_list_widget2.x() - self.final_x_position2
            self.card_list_widget2.move(self.card_list_widget.x() + self.card_list_widget.width() , self.titellabel.height() + 10)
            self.final_x_position2 = self.card_list_widget2.x() - sub
        if (self.card_list_widget2.x() + self.card_list_widget.width() == (self.card_w) * 2 + 40) or (self.card_list_widget2.x() + self.card_list_widget2.width() <= self.card_w * 2 + 40):
            sub = self.card_list_widget.x() - self.final_x_position
            self.card_list_widget.move(self.card_list_widget2.x() + self.card_list_widget2.width() , self.titellabel.height() + 10)
            self.final_x_position = self.card_list_widget.x() - sub
        x = self.card_list_widget.x()
        x2 = self.card_list_widget2.x()
        if (x > self.final_x_position):
            self.ad_move_left_time.stop()
            self.card_list_widget.hide()
            self.card_list_widget2.hide()
            if (x - 50 <= self.final_x_position):
                self.card_list_widget.move(self.final_x_position, self.titellabel.height() + 10)
            else:
                self.card_list_widget.move(x - 50, self.titellabel.height() + 10)
            if (x2 - 50 <= self.final_x_position2):
                self.card_list_widget2.move(self.final_x_position2, self.titellabel.height()+10)
            else:
                self.card_list_widget2.move(x2 - 50, self.titellabel.height() + 10)
            self.card_list_widget.show()
            self.card_list_widget2.show()
            if(Globals.NOWPAGE == PageStates.HOMEPAGE):
                self.ad_move_left_time.start(1)
        else:
            self.ad_move_left_time.stop()
            if(Globals.NOWPAGE == PageStates.HOMEPAGE):
                if(Globals.NEED_SEND_EXPOSURE == True):
                    if(self.card_list_widget.x() < self.width() or (self.card_list_widget.x()+self.card_list_widget.width() < self.width())):
                        for card in self.card_list_widget.children():
                            if(card.x()+self.card_list_widget.x() <= self.width() and card.x()+self.card_list_widget.x() >= 0):
                                if(card.appmsg.app_id not in self.show_id_list):
                                    self.show_id_list.append(card.appmsg.app_id)
                                    self.mw.save_new_arrivals_exposure(card.appmsg.app_id)
                                else:
                                    b =  [val for val in self.add_card_app_id if val not in self.show_id_list]
                                    if(b == []):
                                        Globals.NEED_SEND_EXPOSURE = False
                                        self.show_id_list.clear()
                                    pass
                    if(self.card_list_widget2.x() < self.width() or (self.card_list_widget2.x()+self.card_list_widget2.width() < self.width())):
                        for card in self.card_list_widget2.children():
                            if(card.x()+self.card_list_widget2.x() <= self.width() and card.x()+self.card_list_widget2.x() >= 0):
                                if (card.appmsg.app_id not in self.show_id_list):
                                    self.show_id_list.append(card.appmsg.app_id)
                                    self.mw.save_new_arrivals_exposure(card.appmsg.app_id)
                                else:
                                    b =  [val for val in self.add_card_app_id if val not in self.show_id_list]
                                    if (b == []):
                                        Globals.NEED_SEND_EXPOSURE = False
                                        self.show_id_list.clear()
                                    pass
                else:
                    self.show_id_list.clear()
            else:
                self.show_id_list.clear()
            # self.final_x_position = 0
            # self.final_x_position2 = 0
            self.card_list_widget.show()
            self.card_list_widget2.show()
            self.start_move_time.start(4000)


class New_APP_Card_Widget(QFrame):
    appmsg = []
    image_url = ""
    clicked_ad_card = pyqtSignal()

    def __init__(self,card_w,card_h, appmsg,appname,path,new_arrivals_widget, parent=None):
        QFrame.__init__(self, parent)
        self.m_parent = parent
        self.new_arrivals_widget = new_arrivals_widget
        self.appmsg = appmsg
        self.appname = appname
        self.card_w = card_w
        self.card_h = card_h
        self.init_ui()
        self.get_ad_background(path)

    def init_ui(self):
        self.resize(self.card_w, self.card_h)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setFocusPolicy(Qt.NoFocus)
        self.setStyleSheet("QFrame{border:0px solid  #ffff00;border-radius: 12px;background-color:transparent}")

        self.icon_label = QLabel(self)
        self.icon_label.resize(self.card_w, self.card_h)
        self.icon_label.move(0, 0)

        self.hover_widget = QWidget(self)
        self.hover_widget.setFixedSize(self.width(), self.card_h)
        self.hover_widget.setWindowFlags(Qt.FramelessWindowHint | Qt.WindowStaysOnTopHint)
        # self.card_list_widget.setAttribute(Qt.WA_TranslucentBackground)
        self.hover_widget.setStyleSheet("QWidget{background: rgba(0, 0, 0, 0.1);border-radius: 12px;}")
        self.hover_widget.move(0, 0)
        self.hover_widget.hide()

    def get_ad_background(self,path):
        imagepath = get_new_arrivals_image(path)
        if (Globals.DEVICE == 1):
            self.icon_label.setStyleSheet("QLabel{border-radius: 12px;}")
            pix2 = QPixmap(imagepath)
            fitpixmap_userIcon = pix2.scaled(self.icon_label.width(), self.icon_label.height(), Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
            fitpixmap_userIcon = self.PixmapToRound(fitpixmap_userIcon, self.width(), self.height())
            self.icon_label.setPixmap(fitpixmap_userIcon)
        else:
            self.icon_label.setStyleSheet("QLabel{border-image:url('" + imagepath + "');border-radius: 12px;}")
        self.icon_label.show()
        self.show()

    def PixmapToRound(self,src,x,y):
        pixmapa = QPixmap(src)
        pixmap = QPixmap(x, y)
        pixmap.fill(Qt.transparent)
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.Antialiasing, True)
        # painter.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        path = QPainterPath()
        rect = QRectF(0,0,x,y)
        path.addRoundedRect(rect, 12.0, 12.0)
        painter.setClipPath(path)
        painter.drawPixmap(0, 0, x, y, pixmapa)

        return pixmap


    # 鼠标进入事件

    def enterEvent(self, event):
        self.status = BUTTON_ENTER
        if((self.m_parent == self.new_arrivals_widget.card_list_widget and self.m_parent.x() == self.new_arrivals_widget.final_x_position) or (self.m_parent == self.new_arrivals_widget.card_list_widget2 and self.m_parent.x() == self.new_arrivals_widget.final_x_position2)):
            self.setGeometry(self.x(), self.y(), self.width(), self.height())
            self.shadow = QGraphicsDropShadowEffect(self)
            self.shadow.setOffset(0, 0)
            self.shadow.setColor(QColor(qRgba(149, 149, 149, 0)))
            self.shadow.setBlurRadius(12)
            self.setGraphicsEffect(self.shadow)
        self.hover_widget.show()

    #
    # 函数：鼠标离开事件
    #

    def leaveEvent(self, event):
        self.status = BUTTON_LEAVE
        if ((self.m_parent == self.new_arrivals_widget.card_list_widget and self.m_parent.x() == self.new_arrivals_widget.final_x_position) or (self.m_parent == self.new_arrivals_widget.card_list_widget2 and self.m_parent.x() == self.new_arrivals_widget.final_x_position2)):
            self.resize(self.width(), self.height())
            self.shadow = QGraphicsDropShadowEffect(self)
            self.shadow.setBlurRadius(0)
            self.shadow.setOffset(0, 0)
            self.setGraphicsEffect(self.shadow)
        self.hover_widget.hide()
    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.status = BUTTON_PRESSED
            self.clicked_ad_card.emit()
        else:
            event.ignore()
